#pragma once

#include <QAbstractListModel>
#include <QNetworkAccessManager>
#include <qcontainerfwd.h>
#include <set>

#include <QJsonObject>
#include <QJsonDocument>
#include <QString>
#include <QList>
#include "EventInfo.h"

using namespace std;

class SportModel : public QAbstractListModel
{
  Q_OBJECT

  Q_PROPERTY(QString discipline READ discipline WRITE setDiscipline NOTIFY disciplineChanged);

public:
  enum Role
  {
    EventName = Qt::UserRole + 1,
    Competitors
  };

  explicit SportModel(QObject *parent = nullptr);

  void setDiscipline(QJsonObject discipline)
  {
      this->o_discipline = QJsonObject(discipline);
  }

  virtual int rowCount(const QModelIndex &parent) const override;
  virtual QVariant data(const QModelIndex &index, int role) const override;
  virtual QHash<int, QByteArray> roleNames() const override;

  set<QString> getCategories();
  map<QString, map<QString, int>> getMedalsOfCompetitors();

  // filter to change the current competitor list
  void lastName(QList<Competitor*> &competitors);
  void filterByName(QList<Competitor*> &competitors, QString name);
  void filterByCountry(QList<Competitor*> &competitors, QString nocShort);

  // sort functions to change the order of the current competitor list
//  void sortByName(QList<Competitor*> &competitors);
//  void sortByCountry(QList<Competitor*> &competitors);
//  void sortByResult(QList<Competitor*> &competitors);
//  void sortByMedals(QList<Competitor*> &competitors);
  void reverseOrder(QList<Competitor*> &competitors);

  // statistic function
  void addRelativeToFirst(QList<Competitor*> &competitors);

  QString discipline() const;
  void setDiscipline(const QString &discipline);

public slots:
  void request(QString discipline);
  void parseData();

signals:
    void disciplineChanged();

private:
  QList<EventInfo *> m_sportList;
  QString m_discipline;
  QNetworkAccessManager m_networkManager;
  QNetworkReply *m_reply = nullptr;

  // data from api
  QJsonObject o_discipline;
  bool validateDiscipline();

  void filterCompetitors(QList<Competitor*> &competitors, QString filter); // TODO ref instead of obj

  // function for statistic calculation
  float calcRelativeStat(QString ref, QString val);

};
